from firebase_admin.messaging import send, send_all, send_multicast


class Message:
    def __init__(self, dry_run=False, app=None):
        self._dry_run = dry_run
        self._app = app
        self._message = Message()

    def data(self, data):
        # data: A dictionary of data fields (optional). All keys and values in the dictionary must be
        #             strings.
        self._message.data = data
        return self

    def notification(self, notification):
        # notification: An instance of ``messaging.Notification`` (optional).
        self._message.notification = notification
        return self

    def android(self, android):
        # android: An instance of ``messaging.AndroidConfig`` (optional).
        self._message.android = android
        return self

    def webpush(self, webpush):
        # webpush: An instance of ``messaging.WebpushConfig`` (optional).
        self._message.webpush = webpush
        return self

    def apns(self, apns):
        # apns: An instance of ``messaging.ApnsConfig`` (optional).
        self._message.apns = apns
        return self

    def fcm_options(self, fcm_options):
        # fcm_options: An instance of ``messaging.FCMOptions`` (optional).
        self._message.fcm_options = fcm_options
        return self

    def token(self, token):
        # token: The registration token of the device to which the message should be sent (optional).
        self._message.token = token
        return self

    def topic(self, topic):
        # topic: Name of the FCM topic to which the message should be sent (optional). Topic name
        #             may contain the ``/topics/`` prefix.
        self._message.topic = topic
        return self

    def condition(self, condition):
        # condition: The FCM condition to which the message should be sent (optional).
        self._message.condition = condition
        return self

    def build_send(self):
        return send(message=self._message,
                    dry_run=self._dry_run,
                    app=self._app)


class MulticastMessage:
    def __init__(self, dry_run=False, app=None):
        self.FCM_MAX_RECIPIENTS = 500
        self._tokens = []
        self._dry_run = dry_run
        self._app = app
        self._multicast_message = MulticastMessage()

    def append_token(self, token):
        self._tokens.append(token)

    def replace_tokens(self, tokens):
        self._tokens = tokens

    def data(self, data):
        # data: A dictionary of data fields (optional). All keys and values in the dictionary must be
        #             strings.
        self._multicast_message.data = data
        return self

    def notification(self, notification):
        # notification: An instance of ``messaging.Notification`` (optional).
        self._multicast_message.notification = notification
        return self

    def android(self, android):
        # android: An instance of ``messaging.AndroidConfig`` (optional).
        self._multicast_message.android = android
        return self

    def webpush(self, webpush):
        # webpush: An instance of ``messaging.WebpushConfig`` (optional).
        self._multicast_message.webpush = webpush
        return self

    def apns(self, apns):
        # apns: An instance of ``messaging.ApnsConfig`` (optional).
        self._multicast_message.apns = apns
        return self

    def fcm_options(self, fcm_options):
        # fcm_options: An instance of ``messaging.FCMOptions`` (optional).
        self._multicast_message.fcm_options = fcm_options
        return self

    def multicast_chunks_generator(self):
        for i in range(0, len(self._tokens), self.FCM_MAX_RECIPIENTS):
            self._multicast_message.tokens = self._tokens[i:i + self.FCM_MAX_RECIPIENTS]
            batch_response = send_multicast(multicast_message=self._multicast_message,
                                            dry_run=self._dry_run,
                                            app=self._app)

            yield BatchResponse(self._multicast_message.tokens, batch_response)


class BatchResponse:

    def __init__(self, tokens, batch_response):
        self._tokens = tokens
        self._batch_response = batch_response

    @property
    def failed_tokens(self):
        if self._batch_response.failure_count > 0:
            responses = self._batch_response.responses
            failed_tokens = []
            for index, resp in enumerate(responses):
                if not resp.success:
                    failed_tokens.append(self._tokens[index])
            return failed_tokens
        else:
            return []

    @property
    def failed_count(self):
        return self._batch_response.failure_count


# request error
# - exceptions.DeadlineExceededError
# - exceptions.UnavailableError
# - exceptions.UnknownError
# response error
# -


from firebase_admin.messaging import AndroidNotification
from flask_fcmadmin.cloud_messaging.utils.priority_type import Priority
from flask_fcmadmin.cloud_messaging.utils.visibility_type import Visibility


class AndroidNotificationConfigBuilder:

    def __init__(self):
        self._android_notification_config = AndroidNotification()

    def build(self):
        return self._android_notification_config


class AndroidNotificationTitleBuilder(AndroidNotificationConfigBuilder):
    # title: Title of the notification (optional). If specified, overrides the title set via
    #             ``messaging.Notification``.
    def android_notification_title(self, title):
        self._android_notification_config.title = title
        return self


class AndroidNotificationBodyBuilder(AndroidNotificationTitleBuilder):
    # body: Body of the notification (optional). If specified, overrides the body set via
    #             ``messaging.Notification``.
    def android_notification_body(self, body):
        self._android_notification_config.body = body
        return self


class AndroidNotificationIconBuilder(AndroidNotificationBodyBuilder):
    # icon: Icon of the notification (optional)
    def android_notification_icon(self, icon):
        self._android_notification_config.icon = icon
        return self


class AndroidNotificationColorBuilder(AndroidNotificationIconBuilder):
    # color: Color of the notification icon expressed in ``#rrggbb`` form (optional).
    def android_notification_color(self, color):
        self._android_notification_config.color = color
        return self


class AndroidNotificationSoundBuilder(AndroidNotificationColorBuilder):
    # sound: Sound to be played when the device receives the notification (optional). This is
    #             usually the file name of the sound resource.
    def android_notification_sound(self, sound):
        self._android_notification_config.sound = sound
        return self


class AndroidNotificationTagBuilder(AndroidNotificationSoundBuilder):
    # tag: Tag of the notification (optional). This is an identifier used to replace existing
    #             notifications in the notification drawer. If not specified, each request creates a new
    #             notification.
    def android_notification_tag(self, tag):
        self._android_notification_config.tag = tag
        return self


class AndroidNotificationClickActionBuilder(AndroidNotificationTagBuilder):
    # click_action: The action associated with a user click on the notification (optional). If
    #             specified, an activity with a matching intent filter is launched when a user clicks on
    #             the notification.
    def android_notification_click_action(self, click_action):
        self._android_notification_config.click_action = click_action
        return self


class AndroidNotificationBodyLocKeyBuilder(AndroidNotificationClickActionBuilder):
    # body_loc_key: Key of the body string in the app's string resources to use to localize the
    #             body text (optional).
    def android_notification_body_loc_key(self, body_loc_key):
        self._android_notification_config.body_loc_key = body_loc_key
        return self


class AndroidNotificationBodyLocArgsBuilder(AndroidNotificationBodyLocKeyBuilder):
    # body_loc_args: A list of resource keys that will be used in place of the format specifiers
    #             in ``body_loc_key`` (optional).
    def android_notification_body_loc_args(self, body_loc_args):
        self._android_notification_config.body_loc_args = body_loc_args
        return self


class AndroidNotificationTitleLocKeyBuilder(AndroidNotificationBodyLocArgsBuilder):
    # title_loc_key: Key of the title string in the app's string resources to use to localize the
    #             title text (optional).
    def android_notification_title_loc_key(self, title_loc_key):
        self._android_notification_config.title_loc_key = title_loc_key
        return self


class AndroidNotificationTitleLocArgsBuilder(AndroidNotificationTitleLocKeyBuilder):
    # title_loc_args: A list of resource keys that will be used in place of the format specifiers
    #             in ``title_loc_key`` (optional).
    def android_notification_title_loc_args(self, title_loc_args):
        self._android_notification_config.title_loc_args = title_loc_args
        return self


class AndroidNotificationChannelIDBuilder(AndroidNotificationTitleLocArgsBuilder):
    # channel_id: channel_id of the notification (optional).
    def android_notification_channel_id(self, channel_id):
        self._android_notification_config.channel_id = channel_id
        return self


class AndroidNotificationImageBuilder(AndroidNotificationChannelIDBuilder):
    # image: Image url of the notification (optional).
    def android_notification_image(self, image):
        self._android_notification_config.image = image
        return self


class AndroidNotificationTickerBuilder(AndroidNotificationImageBuilder):
    # ticker: Sets the ``ticker`` text, which is sent to accessibility services. Prior to API
    #             level 21 (Lollipop), sets the text that is displayed in the status bar when the
    #             notification first arrives (optional).
    def android_notification_ticker(self, ticker):
        self._android_notification_config.ticker = ticker
        return self


class AndroidNotificationStickyBuilder(AndroidNotificationTickerBuilder):
    # sticky: When set to ``False`` or unset, the notification is automatically dismissed when the
    #             user clicks it in the panel. When set to ``True``, the notification persists even when
    #             the user clicks it (optional).
    def android_notification_sticky(self, sticky):
        self._android_notification_config.sticky = sticky
        return self


class AndroidNotificationEventTimestampBuilder(AndroidNotificationStickyBuilder):
    # event_timestamp: For notifications that inform users about events with an absolute time
    #             reference, sets the time that the event in the notification occurred as a
    #             ``datetime.datetime`` instance. If the ``datetime.datetime`` instance is naive, it
    #             defaults to be in the UTC timezone. Notifications in the panel are sorted by this time
    #             (optional).
    def android_notification_event_timestamp(self, event_timestamp):
        self._android_notification_config.event_timestamp = event_timestamp
        return self


class AndroidNotificationLocalOnlyBuilder(AndroidNotificationEventTimestampBuilder):
    # local_only: Sets whether or not this notification is relevant only to the current device.
    #             Some notifications can be bridged to other devices for remote display, such as a Wear OS
    #             watch. This hint can be set to recommend this notification not be bridged (optional).
    #             See Wear OS guides:
    #             https://developer.android.com/training/wearables/notifications/bridger#existing-method-of-preventing-bridging
    def android_notification_local_only(self, local_only):
        self._android_notification_config.local_only = local_only
        return self



class AndroidNotificationPriorityBuilder(AndroidNotificationLocalOnlyBuilder):
    # priority: Sets the relative priority for this notification. Low-priority notifications may
    #             be hidden from the user in certain situations. Note this priority differs from
    #             ``AndroidMessagePriority``. This priority is processed by the client after the message
    #             has been delivered. Whereas ``AndroidMessagePriority`` is an FCM concept that controls
    #             when the message is delivered (optional). Must be one of ``default``, ``min``, ``low``,
    #             ``high``, ``max`` or ``normal``.
    def android_notification_default_priority(self):
        self._android_notification_config.priority = Priority.DEFAULT.value
        return self

    def android_notification_min_priority(self):
        self._android_notification_config.priority = Priority.MIN.value
        return self

    def android_notification_low_priority(self):
        self._android_notification_config.priority = Priority.LOW.value
        return self

    def android_notification_high_priority(self):
        self._android_notification_config.priority = Priority.HIGH.value
        return self

    def android_notification_max_priority(self):
        self._android_notification_config.priority = Priority.MAX.value
        return self

    def android_notification_normal_priority(self):
        self._android_notification_config.priority = Priority.NORMAL.value
        return self


class AndroidNotificationVibrateTimingsMillisBuilder(AndroidNotificationPriorityBuilder):
    # vibrate_timings_millis: Sets the vibration pattern to use. Pass in an array of milliseconds
    #             to turn the vibrator on or off. The first value indicates the duration to wait before
    #             turning the vibrator on. The next value indicates the duration to keep the vibrator on.
    #             Subsequent values alternate between duration to turn the vibrator off and to turn the
    #             vibrator on. If ``vibrate_timings`` is set and ``default_vibrate_timings`` is set to
    #             ``True``, the default value is used instead of the user-specified ``vibrate_timings``.
    def android_notification_vibrate_timings_millis(self, vibrate_timings_millis):
        self._android_notification_config.vibrate_timings_millis = vibrate_timings_millis
        return self


class AndroidNotificationDefaultVibrateTimingsBuilder(AndroidNotificationPriorityBuilder):
    # default_vibrate_timings: If set to ``True``, use the Android framework's default vibrate
    #             pattern for the notification (optional). Default values are specified in ``config.xml``
    #             https://android.googlesource.com/platform/frameworks/base/+/master/core/res/res/values/config.xml.
    #             If ``default_vibrate_timings`` is set to ``True`` and ``vibrate_timings`` is also set,
    #             the default value is used instead of the user-specified ``vibrate_timings``.
    def android_notification_default_vibrate_timings(self, default_vibrate_timings):
        self._android_notification_config.default_vibrate_timings = default_vibrate_timings
        return self


class AndroidNotificationDefaultSoundBuilder(AndroidNotificationDefaultVibrateTimingsBuilder):
    # default_sound: If set to ``True``, use the Android framework's default sound for the
    #             notification (optional). Default values are specified in ``config.xml``
    #             https://android.googlesource.com/platform/frameworks/base/+/master/core/res/res/values/config.xml
    def android_notification_default_sound(self, default_sound):
        self._android_notification_config.default_sound = default_sound
        return self


class AndroidNotificationLightSettingsBuilder(AndroidNotificationDefaultSoundBuilder):
    # light_settings: Settings to control the notification's LED blinking rate and color if LED is
    #             available on the device. The total blinking time is controlled by the OS (optional).
    def android_notification_light_settings(self, light_settings):
        self._android_notification_config.light_settings = light_settings
        return self


class AndroidNotificationDefaultLightSettingsBuilder(AndroidNotificationLightSettingsBuilder):
    # default_light_settings: If set to ``True``, use the Android framework's default LED light
    #             settings for the notification. Default values are specified in ``config.xml``
    #             https://android.googlesource.com/platform/frameworks/base/+/master/core/res/res/values/config.xml.
    #             If ``default_light_settings`` is set to ``True`` and ``light_settings`` is also set, the
    #             user-specified ``light_settings`` is used instead of the default value.
    def android_notification_default_light_settings(self, default_light_settings):
        self._android_notification_config.default_light_settings = default_light_settings
        return self


class AndroidNotificationVisibilityBuilder(AndroidNotificationDefaultLightSettingsBuilder):
    # visibility: Sets the visibility of the notification. Must be either ``private``, ``public``,
    #             or ``secret``. If unspecified, default to ``private``.
    def android_notification_private_visibility(self):
        self._android_notification_config.visibility = Visibility.PRIVATE.value
        return self

    def android_notification_public_visibility(self):
        self._android_notification_config.visibility = Visibility.PUBLIC.value
        return self

    def android_notification_secret_visibility(self):
        self._android_notification_config.visibility = Visibility.SECRET.value
        return self


class AndroidNotificationNotificationCountBuilder(AndroidNotificationVisibilityBuilder):
    # notification_count: Sets the number of items this notification represents. May be displayed
    #             as a badge count for Launchers that support badging. See ``NotificationBadge``
    #             https://developer.android.com/training/notify-user/badges. For example, this might be
    #             useful if you're using just one notification to represent multiple new messages but you
    #             want the count here to represent the number of total new messages. If zero or
    #             unspecified, systems that support badging use the default, which is to increment a
    #             number displayed on the long-press menu each time a new notification arrives.
    def android_notification_notification_count(self, notification_count):
        self._android_notification_config.notification_count = notification_count
        return self


class AndroidNotificationBuilder(AndroidNotificationNotificationCountBuilder):

    @classmethod
    def config(cls):
        return AndroidNotificationBuilder()




from firebase_admin.messaging import FCMOptions


class FCMOptionsConfigBuilder:
    def __init__(self):
        self._fcm_options = FCMOptions()

    def build(self):
        return self._fcm_options


class FCMOptionsAnalyticsLabelBuilder(FCMOptionsConfigBuilder):

    def analytics_label(self, analytics_label=None):
        self._fcm_options.analytics_label = analytics_label
        return self


class FCMOptionsBuilder(FCMOptionsAnalyticsLabelBuilder):

    @classmethod
    def config(cls):
        return FCMOptionsBuilder()


from firebase_admin.messaging import Aps

# Aps dictionary to be included in an APNS payload
class APNSApsConfigBuilder:
    def __init__(self):
        self._aps_config = Aps()

    def build(self):
        return self._aps_config


class APNSApsAlertConfig(APNSApsConfigBuilder):
    # alert: A string or a ``messaging.ApsAlert`` instance (optional).
    def alert(self, alert):
        self._aps_config.alert = alert
        return self


class APNSApsBadgeConfig(APNSApsAlertConfig):
    # badge: A number representing the badge to be displayed with the message (optional).
    def badge(self, badge):
        self._aps_config.badge = badge
        return self


class APNSApsSoundConfig(APNSApsBadgeConfig):
    # sound: Name of the sound file to be played with the message or a
    #             ``messaging.CriticalSound`` instance (optional).
    def sound(self, sound):
        self._aps_config.sound = sound
        return self


class APNSApsContentAvailableConfig(APNSApsSoundConfig):
    # content_available: A boolean indicating whether to configure a background update
    #             notification (optional).
    def content_available(self, content_available):
        self._aps_config.content_available = content_available
        return self


class APNSApsCategoryConfig(APNSApsContentAvailableConfig):
    # category: String identifier representing the message type (optional).
    def category(self, category):
        self._aps_config.category = category
        return self


class APNSApsThreadIDConfig(APNSApsCategoryConfig):
    # thread_id: An app-specific string identifier for grouping messages (optional).
    def thread_id(self, thread_id):
        self._aps_config.thread_id = thread_id
        return self


class APNSApsMutableContentConfig(APNSApsThreadIDConfig):
    #  mutable_content: A boolean indicating whether to support mutating notifications at
    #             the client using app extensions (optional).
    def mutable_content(self, mutable_content):
        self._aps_config.mutable_content = mutable_content
        return self


class APNSApsCustomDataConfig(APNSApsMutableContentConfig):
    # custom_data: A dict of custom key-value pairs to be included in the Aps dictionary
    #             (optional).
    def custom_data(self, custom_data):
        self._aps_config.custom_data = custom_data
        return self


class APNSApsBuilder(APNSApsCustomDataConfig):

    @classmethod
    def config(cls):
        return APNSApsBuilder()



from firebase_admin import credentials
import firebase_admin


class FlaskFCMCenter:

    def __init__(self, prefix="FCM_ADMIN", instance_name="DEFAULT"):
        self._firebase_admin = None
        self.prefix = prefix
        self.instance_name = instance_name
        self._file = None

    def init_app(self, app, file):
        # Get flask app config file content
        # config = app.config
        cred = credentials.Certificate(file)
        self._file = file
        self._firebase_admin = firebase_admin.initialize_app(credential=cred, name=self.instance_name.lower())

        if not hasattr(app, "extensions"):
            app.extensions = {}
        app.extensions[f"{self.prefix}_{self.instance_name}".lower()] = self

    def __getattr__(self, name):
        return getattr(self._firebase_admin, name)

    def __getitem__(self, name):
        return self._firebase_admin[name]

    @property
    def app(self):
        return self._firebase_admin



from firebase_admin.messaging import LightSettings


class LightSettingsConfigBuilder:
    def __init__(self, color, light_on_millis, light_off_mills):
        self._light_settings_config = LightSettings(color, light_on_millis, light_off_mills)

    def build(self):
        return self._light_settings_config


class LightSettingsBuilder(LightSettingsConfigBuilder):

    # color: Sets the color of the LED in ``#rrggbb`` or ``#rrggbbaa`` format.
    # light_on_duration_millis: Along with ``light_off_duration``, defines the blink rate of LED
    #             flashes.
    # light_on_duration_millis: Along with ``light_off_duration``, defines the blink rate of LED
    #             flashes.
    @classmethod
    def config_light_settings(cls, color, light_on_millis, light_off_mills):
        return LightSettingsBuilder(color, light_on_millis, light_off_mills)




from firebase_admin.messaging import AndroidFCMOptions


class AndroidFCMOptionsConfigBuilder:
    def __init__(self):
        self._android_fcm_options = AndroidFCMOptions()

    def build(self):
        return self._android_fcm_options


class AndroidFCMOptionsAnalyticsLabelBuilder(AndroidFCMOptionsConfigBuilder):

    def analytics_label(self, analytics_label=None):
        self._android_fcm_options.analytics_label = analytics_label
        return self


class AndroidFCMOptionsBuilder(AndroidFCMOptionsAnalyticsLabelBuilder):

    @classmethod
    def config(cls):
        return AndroidFCMOptionsBuilder()






from firebase_admin.messaging import APNSConfig


class APNSConfigBuilder:
    def __init__(self):
        self._apns_config = APNSConfig()

    def build(self):
        return self._apns_config


class APNSHeadersConfig(APNSConfigBuilder):
    # headers: A dictionary of headers (optional).
    def headers(self, headers):
        self._apns_config.headers = headers
        return self


class APNSPayloadConfig(APNSHeadersConfig):
    # payload: A ``messaging.APNSPayload`` to be included in the message (optional).
    def payload(self, payload):
        self._apns_config.payload = payload
        return self


class APNSFCMOptionsConfig(APNSPayloadConfig):
    # fcm_options: A ``messaging.APNSFCMOptions`` instance to be included in the message
    #       (optional).
    def fcm_options(self, fcm_options):
        self._apns_config.fcm_options = fcm_options
        return self


class APNSBuilder(APNSFCMOptionsConfig):

    @classmethod
    def config(cls):
        return APNSBuilder()



from firebase_admin.messaging import ApsAlert


# An alert that can be included in ``messaging.Aps``.
class APNSApsApsAlertConfigBuilder:
    def __init__(self):
        self._aps_alert = ApsAlert()

    def build(self):
        return self._aps_alert


class APNSApsAlertTitleBuilder(APNSApsApsAlertConfigBuilder):
    # title: Title of the alert (optional). If specified, overrides the title set via
    #             ``messaging.Notification``.
    def title(self, title):
        self._aps_alert.title = title
        return self


class APNSApsAlertSubtitleBuilder(APNSApsAlertTitleBuilder):
    # subtitle: Subtitle of the alert (optional).
    def subtitle(self, subtitle):
        self._aps_alert.critical = subtitle
        return self


class APNSApsAlertBodyBuilder(APNSApsAlertSubtitleBuilder):
    # body: Body of the alert (optional). If specified, overrides the body set via
    #             ``messaging.Notification``.
    def body(self, body):
        self._aps_alert.body = body
        return self


class APNSApsAlertLocKeyBuilder(APNSApsAlertBodyBuilder):
    # loc_key: Key of the body string in the app's string resources to use to localize the
    #             body text (optional).
    def loc_key(self, loc_key):
        self._aps_alert.loc_key = loc_key
        return self


class APNSApsAlertLocArgsBuilder(APNSApsAlertLocKeyBuilder):
    # loc_args: A list of resource keys that will be used in place of the format specifiers
    #             in ``loc_key`` (optional).
    def loc_args(self, loc_args):
        self._aps_alert.loc_args = loc_args
        return self


class APNSApsAlertTitleLocKeyBuilder(APNSApsAlertLocArgsBuilder):
    # title_loc_key: Key of the title string in the app's string resources to use to localize the
    #             title text (optional).
    def title_loc_key(self, title_loc_key):
        self._aps_alert.title_loc_key = title_loc_key
        return self


class APNSApsAlertTitleLocArgsBuilder(APNSApsAlertTitleLocKeyBuilder):
    # title_loc_args: A list of resource keys that will be used in place of the format specifiers
    #             in ``title_loc_key`` (optional).
    def title_loc_args(self, title_loc_args):
        self._aps_alert.title_loc_args = title_loc_args
        return self


class APNSApsAlertActionLocKeyBuilder(APNSApsAlertTitleLocArgsBuilder):
    # action_loc_key: Key of the text in the app's string resources to use to localize the
    #             action button text (optional).
    def action_loc_key(self, action_loc_key):
        self._aps_alert.action_loc_key = action_loc_key
        return self


class APNSApsAlertLaunchImageBuilder(APNSApsAlertActionLocKeyBuilder):
    # launch_image: Image for the notification action (optional).
    def launch_image(self, launch_image):
        self._aps_alert.launch_image = launch_image
        return self


class APNSApsAlertCustomDataBuilder(APNSApsAlertLaunchImageBuilder):
    # custom_data: A dict of custom key-value pairs to be included in the ApsAlert dictionary
    #             (optional)
    def custom_data(self, custom_data):
        self._aps_alert.custom_data = custom_data
        return self


class APNSApsApsAlertBuilder(APNSApsAlertCustomDataBuilder):

    @classmethod
    def config(cls):
        return APNSApsApsAlertBuilder()



from firebase_admin.messaging import APNSFCMOptions


class APNSFCMOptionsConfigBuilder:
    def __init__(self):
        self._apns_fcm_options = APNSFCMOptions()

    def build(self):
        return self._apns_fcm_options


class APNSFCMOptionsAnalyticsLabelBuilder(APNSFCMOptionsConfigBuilder):

    def analytics_label(self, analytics_label=None):
        self._apns_fcm_options.analytics_label = analytics_label
        return self


class APNSFCMOptionsBuilder(APNSFCMOptionsAnalyticsLabelBuilder):

    @classmethod
    def config(cls):
        return APNSFCMOptionsBuilder()



from firebase_admin.messaging import APNSPayload


class APNSPayloadConfigBuilder:
    def __init__(self):
        self._apns_payload_config = APNSPayload()

    def build(self):
        return self._apns_payload_config


class APNSPayloadAPSBuilder(APNSPayloadConfigBuilder):
    # aps: A ``messaging.Aps`` instance to be included in the payload.
    def aps(self, aps):
        self._apns_payload_config.aps = aps
        return self


class APNSPayloadCustomDataBuilder(APNSPayloadAPSBuilder):
    # kwargs: Arbitrary keyword arguments to be included as custom fields in the payload
    def custom_data(self, **kwargs):
        self._apns_payload_config.custom_data = kwargs
        return self


class APNSPayloadBuilder(APNSPayloadCustomDataBuilder):

    @classmethod
    def config(cls):
        return APNSPayloadBuilder()



from firebase_admin.messaging import CriticalSound


class APNSApsCriticalSoundConfigBuilder:
    def __init__(self):
        self.aps_critical_sound = CriticalSound()

    def build(self):
        return self.aps_critical_sound


class APNSApsNameBuilder(APNSApsCriticalSoundConfigBuilder):
    # name: The name of a sound file in your app's main bundle or in the ``Library/Sounds``
    #             folder of your app's container directory. Specify the string ``default`` to play the
    #             system sound.
    def name(self, name):
        self.aps_critical_sound.name = name
        return self


class APNSApsCriticalBuilder(APNSApsNameBuilder):
    # critical: Set to ``True`` to set the critical alert flag on the sound configuration
    #             (optional).
    def critical(self, critical=None):
        self.aps_critical_sound.critical = critical
        return self


class APNSApsVolumeBuilder(APNSApsCriticalBuilder):
    # volume: The volume for the critical alert's sound. Must be a value between 0.0 (silent)
    #             and 1.0 (full volume) (optional).
    def volume(self, volume=None):
        self.aps_critical_sound.volume = volume
        return self


class APNSApsCriticalSoundBuilder(APNSApsVolumeBuilder):

    @classmethod
    def config(cls):
        return APNSApsCriticalSoundBuilder()



from firebase_admin.messaging import Notification


class NotificationConfigBuilder:
    def __init__(self):
        self._notification_config = Notification()

    def build(self):
        return self._notification_config


class NotificationTitleBuilder(NotificationConfigBuilder):
    # title: Title of the notification (optional)
    def title(self, title):
        self._notification_config.title = title
        return self


class NotificationBodyBuilder(NotificationTitleBuilder):
    # body: Body of the notification (optional)
    def body(self, body):
        self._notification_config.body = body
        return self


class NotificationImageBuilder(NotificationBodyBuilder):
    # image: Image url of the notification (optional)
    def image(self, image):
        self._notification_config.image = image
        return self


class NotificationBuilder(NotificationImageBuilder):

    @classmethod
    def config(cls):
        return NotificationBuilder()




import setuptools

INSTALL_REQUIRES = ["Flask==1.1.2", "firebase-admin==4.5.0"]

setuptools.setup(
    name="flask-FCMAdmin",# Replace with your own username
    version="0.0.7",
    author="chienaeae",
    author_email="chienaeae@gmail.com",
    description="fcm admin with flask",
    long_description=
    '''
    # ALPHA version
    
    fcm admin with flask
    ''',
    long_description_content_type="text/markdown",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3.9",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    install_requires=INSTALL_REQUIRES,
    python_requires='>=3.6',
)


from firebase_admin.messaging import AndroidConfig


class AndroidConfigBuilder:
    def __init__(self):
        self._android_config = AndroidConfig()

    def build(self):
        return self._android_config


class AndroidCollapseKeyBuilder(AndroidConfigBuilder):
    # collapse_key: Collapse key string for the message (optional). This is an identifier for a
    #             group of messages that can be collapsed, so that only the last message is sent when
    #             delivery can be resumed. A maximum of 4 different collapse keys may be active at a
    #             given time.
    def collapse_key(self, collapse_key):
        self._android_config.collapse_key = collapse_key
        return self


class AndroidPriorityBuilder(AndroidCollapseKeyBuilder):
    # priority: Priority of the message (optional). Must be one of ``high`` or ``normal``.
    def priority(self, priority):
        self._android_config.priority = priority
        return self


class AndroidTTLBuilder(AndroidPriorityBuilder):
    # ttl: The time-to-live duration of the message (optional). This can be specified
    #             as a numeric seconds value or a ``datetime.timedelta`` instance.
    def ttl(self, duration):
        self._android_config.ttl = duration
        return self


class AndroidRestrictedPackageNameBuilder(AndroidTTLBuilder):
    # restricted_package_name: The package name of the application where the registration tokens
    #             must match in order to receive the message (optional).
    def restricted_package_name(self, restricted_package_name):
        self._android_config.restricted_package_name = restricted_package_name
        return self


class AndroidDataBuilder(AndroidRestrictedPackageNameBuilder):
    # data: A dictionary of data fields (optional). All keys and values in the dictionary must be
    #             strings. When specified, overrides any data fields set via ``Message.data``.
    def data(self, data):
        self._android_config.data = data
        return self


class AndroidNotificationBuilder(AndroidDataBuilder):
    # notification: A ``messaging.AndroidNotification`` to be included in the message (optional).
    def notification(self, notification):
        self._android_config.notification = notification
        return self


class AndroidFCMOptionsBuilder(AndroidNotificationBuilder):
    # fcm_options: A ``messaging.AndroidFCMOptions`` to be included in the message (optional).
    def fcm_options(self, fcm_options):
        self._android_config.notification = fcm_options
        return self


class AndroidBuilder(AndroidFCMOptionsBuilder):

    @classmethod
    def config(cls):
        return AndroidBuilder()


